import { useState, useEffect } from 'react';
import { randomBernoulli } from 'd3';

const randomGen = randomBernoulli(0.5);

const countNeighbors = (m, x, y) =>
  [
    m[x][y + 1],
    m[x][y - 1],
    m[x + 1] && m[x + 1][y],
    m[x + 1] && m[x + 1][y + 1],
    m[x + 1] && m[x + 1][y - 1],
    m[x - 1] && m[x - 1][y],
    m[x - 1] && m[x - 1][y + 1],
    m[x - 1] && m[x - 1][y - 1],
  ].filter((x) => x === 1).length;

const go = (alive, neighbors) =>
  alive
    ? [2, 3].includes(neighbors)
      ? 1
      : 0
    : neighbors === 3
    ? 1
    : 0;

export const CellMatrix = ({
  nx,
  ny,
  innerWidth,
  innerHeight,
}) => {
  const cellWidth = innerWidth / nx;
  const cellHeight = innerHeight / ny;

  const [matrix, setMatrix] = useState(
    Array(nx)
      .fill()
      .map((_) => Array(ny).fill().map(randomGen))
  );

  useEffect(() => {
    const interval = setInterval(() => {
      let newMatrix = matrix.map((ar, ix) =>
        ar.map((cell, iy) =>
          go(cell, countNeighbors(matrix, ix, iy))
        )
      );
      setMatrix(newMatrix);
    }, 2000);
  }, []);

  return matrix.map((ar, ix) =>
    ar.map((cell, iy) => (
      <rect
        x={ix * cellWidth}
        y={iy * cellHeight}
        width={cellWidth}
        height={cellHeight}
        shapeRendering="optimizeSpeed"
        fill={cell ? '#D8C800' : 'none'}
      />
    ))
  );
};
